#!/usr/bin/env python3
# -*- coding: utf-8 -*-

extensions = [
    "sphinx.ext.doctest",
    "sphinx.ext.todo",
    "sphinx.ext.coverage",
    "sphinx.ext.viewcode",
]

templates_path = ["_templates"]

source_suffix = ".rst"

master_doc = "index"

project = "waf-libs"
copyright = "2019, Felix Wong"
author = "Felix Wong"

version = open("../../VERSION").read().strip()
release = version

language = None

exclude_patterns = []

pygments_style = "sphinx"

todo_include_todos = True

html_theme = "alabaster"

html_static_path = ["_static"]

htmlhelp_basename = "waf-libsdoc"

latex_elements = {
    "papersize": "letterpaper",
    "pointsize": "10pt",
    "preamble": "",
    "figure_align": "htbp",
}

latex_documents = [
    (
        master_doc,
        "waf-libs.tex",
        "waf-libs Documentation",
        "Felix Wong",
        "manual",
    ),
]

man_pages = [(master_doc, "waf-libs", "waf-libs Documentation", [author], 1)]

texinfo_documents = [
    (
        master_doc,
        "waf-libs",
        "waf-libs Documentation",
        author,
        "waf-libs",
        "One line description of project.",
        "Miscellaneous",
    ),
]



"""
text utils
"""

import sys

from waflibs import log

logger = log.logger().get_logger()

TIMES = 77
logger.debug(f"default number of times is {TIMES}")


def divider(char="=", times=TIMES):
    """generate text divider"""

    return char * times


def print_divider(char="=", times=TIMES, stderr=False):
    """print text divider"""

    output = divider(char, times)

    if stderr:
        print(output, file=sys.stderr)
    else:
        print(output)


def stderrprint(*args, **kwargs):
    logger.debug(f"original kwargs: {kwargs}")

    output = sys.stderr
    if "stdout" in kwargs and kwargs["stdout"]:
        output = sys.stdout
        kwargs.pop("stdout")
    logger.debug(f"file output is {output.name}")

    if "newline" in kwargs and not kwargs["newline"]:
        kwargs["flush"] = True
        kwargs["end"] = ""
        kwargs.pop("newline")
    logger.debug(f"after kwargs: {kwargs}")

    print(*args, **kwargs, file=output)


eprint = stderrprint
errprint = stderrprint
errorprint = stderrprint



import bitwarden_sdk

from waflibs import log

"""
bitwarden secrets manager
"""

logger = log.logger().get_logger()

API_URL = "https://api.bitwarden.com"
IDENTITY_URL = "https://identity.bitwarden.com"


class client:
    def __init__(
        self, access_token, org_id, api_url=API_URL, identity_url=IDENTITY_URL
    ):
        logger.debug(f"api url: {api_url}")
        logger.debug(f"identity url: {identity_url}")

        self.org_id = org_id
        self.client = bitwarden_sdk.BitwardenClient(
            bitwarden_sdk.client_settings_from_dict(
                {
                    "apiUrl": api_url,
                    "deviceType": bitwarden_sdk.DeviceType.SDK,
                    "identityUrl": identity_url,
                    "userAgent": "waflibs Python",
                }
            )
        )

        self.client.access_token_login(access_token)

    def get_secret(self, secret_name):
        logger.debug(f"secret name: {secret_name}")

        secrets = self.client.secrets().list(self.org_id).data.to_dict()
        logger.debug(f"all secrets: {secrets}")
        for secret in secrets["data"]:
            if secret["key"] == secret_name:
                logger.debug(f"secret to return: {secret}")

                secret_obj = self.client.secrets().get(secret["id"])
                logger.debug(f"secret object: {secret_obj}")
                return secret_obj.data.value



#!/usr/bin/python
"""helpers to manipulate config file"""

import copy
import json
import pathlib

import yaml
from waflibs import log

logger = log.logger().get_logger()


def sanitize(config, deep_copy=True):
    """remove secrets from config file"""

    if deep_copy:
        config_copy = copy.deepcopy(config)
    else:
        config_copy = config

    secret_text = [
        "password",
        "passwords",
        "api-token",
        "api-tokens",
        "api_token",
        "api_tokens",
        "api-key",
        "api-keys",
        "api_key",
        "api_keys",
        "token",
        "tokens",
    ]

    if hasattr(config_copy, "items"):
        config_dict = config_copy
    else:
        config_dict = vars(config_copy)
    for k, v in config_dict.items():
        if isinstance(v, dict):
            sanitize(v, deep_copy=False)
        else:
            for text in secret_text:
                if text in k:
                    config_dict[k] = "REDACTED"

    return config_dict


def get_config_filename(fname, ignore_suffix=False):
    """get config filename"""

    logger.debug(f"config filename: {fname}")

    file = pathlib.Path(fname)
    if ignore_suffix:
        logger.debug("ignore_suffix flag set to true... ignoring suffix")
    else:
        exts = [
            "yml",
            "yaml",
        ]

        for ext in exts:
            suffix = f".{ext}"
            logger.debug(f"suffix: {suffix}")
            file_suffix = file.suffix
            logger.debug(f"file suffix: {file_suffix}")

            if file_suffix == suffix:
                file = file.with_suffix(suffix)

                logger.debug(f"filename: {file}")

    return file


def parse_yaml_file(fname, ignore_suffix=False):
    """parse yaml config by filename"""

    filename = get_config_filename(fname, ignore_suffix)

    if not filename:
        raise Exception(f"no yaml config file found at {fname}")
    else:
        with open(pathlib.Path(filename)) as f:
            config = parse_yaml(f.read())

        return config


def parse_yaml(config):
    """parse yaml config"""

    return yaml.load(config, Loader=yaml.FullLoader)


def parse_json_file(filename):
    """parse json config by filename"""

    f = open(filename)
    config = parse_json(f.read())
    f.close()

    return config


def parse_json(config):
    """parse json config"""

    return json.loads(config)



#!/usr/bin/env python

import random
import string
import tempfile
import unittest

from waflibs import config


def random_word(length=29):
    return "".join(random.choice(string.ascii_lowercase) for i in range(length))


class TestConfig(unittest.TestCase):
    pass


class TestParseConfig(TestConfig):
    json = b"""
{
  "a": "b",
  "c": {
    "d": "e"
  },
  "f": {
    "g": {
      "h": "i"
    }
  }
}
"""

    yaml = b"""
---
a: b
c:
  d: e
f:
  g:
    h: i
"""

    result_yaml_dict = {"a": "b", "c": {"d": "e"}, "f": {"g": {"h": "i"}}}
    result_json_dict = {"a": "b", "c": {"d": "e"}, "f": {"g": {"h": "i"}}}

    def test_parse_yaml(self):
        self.assertEqual(config.parse_yaml(self.yaml), self.result_yaml_dict)

    def test_parse_yaml_file_error(self):
        with tempfile.NamedTemporaryFile() as f:
            f.write(b"ignore")
            f.seek(0)

            with self.assertRaises(Exception):
                config.parse_yaml_file(f.name)

    def test_parse_bad_yaml_ext_file(self):
        with tempfile.NamedTemporaryFile(suffix=".bad") as f:
            f.write(self.yaml)
            f.seek(0)

            with self.assertRaises(Exception):
                config.parse_yaml_file(f.name)

    def test_parse_yaml_ext_file(self):
        with tempfile.NamedTemporaryFile(suffix=".yaml") as f:
            f.write(self.yaml)
            f.seek(0)

            self.assertEqual(config.parse_yaml_file(f.name), self.result_yaml_dict)

    def test_parse_json(self):
        self.assertEqual(config.parse_json(self.json), self.result_json_dict)

    def test_parse_json_file(self):
        with tempfile.NamedTemporaryFile() as f:
            f.write(self.json)
            f.seek(0)

            self.assertEqual(config.parse_json_file(f.name), self.result_json_dict)


class TestSanitizeConfig(TestConfig):
    def test_orig_dict_not_modified(self):
        orig_dict = {
            "real": "should not change",
            "token": "kozxokawekowaefokijwef",
        }
        result_dict = {"real": "should not change", "token": "REDACTED"}

        self.assertEqual(config.sanitize(orig_dict), result_dict)

    def test_empty_dict(self):
        orig_dict = {}
        result_dict = {}

        self.assertEqual(config.sanitize(orig_dict), result_dict)

    def test_single_dict(self):
        orig_dict = {
            "merp_token": "walfj",
            "fail_password": "lwfjealkwef",
            "password": "supersecretalfjeawoefjk",
            "token": "awfjeoawkfj",
            "real": "should not change",
        }
        result_dict = {
            "merp_token": "REDACTED",
            "fail_password": "REDACTED",
            "password": "REDACTED",
            "token": "REDACTED",
            "real": "should not change",
        }

        self.assertEqual(config.sanitize(orig_dict), result_dict)

    def test_single_multi_dict(self):
        orig_dict = {"asdf": {"password": "laksdjflkasdjfk"}}
        result_dict = {"asdf": {"password": "REDACTED"}}

        self.assertEqual(config.sanitize(orig_dict), result_dict)

    def test_single_multi_dict_entries(self):
        orig_dict = {
            "asdf": {
                "password": "laksdjflkasdjfk",
                "more_token": "zxocvuoxcbiuzoxcibu",
            }
        }
        result_dict = {"asdf": {"password": "REDACTED", "more_token": "REDACTED"}}

        self.assertEqual(config.sanitize(orig_dict), result_dict)

    def test_multi_multi_dict(self):
        orig_dict = {
            "asdf": {
                "password": "laksdjflkasdjfk",
                "more_token": "zxocvuoxcbiuzoxcibu",
            },
            "oiuzxc": {
                "tokens": "uoyioiyuuoiot",
                "real": "should not change",
            },
        }
        result_dict = {
            "asdf": {"password": "REDACTED", "more_token": "REDACTED"},
            "oiuzxc": {"tokens": "REDACTED", "real": "should not change"},
        }

        self.assertEqual(config.sanitize(orig_dict), result_dict)

    def test_multi_tokens_dict(self):
        orig_dict = {
            "asdf": {
                "password": random_word(),
                "more_token": random_word(),
            },
            "oiuzxc": {
                "tokens": [
                    random_word(),
                    random_word(),
                ],
                "real": "should not change",
                "token": random_word(),
            },
        }
        result_dict = {
            "asdf": {"password": "REDACTED", "more_token": "REDACTED"},
            "oiuzxc": {
                "tokens": "REDACTED",
                "real": "should not change",
                "token": "REDACTED",
            },
        }

        self.assertEqual(config.sanitize(orig_dict), result_dict)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

import argparse
import unittest

from waflibs import log


class TestLog(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        v_parser = argparse.ArgumentParser(description="test verbose logger")
        v_parser.add_argument("-v", "--verbose", action="store_true", help="verbose")
        cls.v_args = v_parser.parse_args(["-v"])

        parser = argparse.ArgumentParser(description="test logger")
        cls.args = parser.parse_args({})


class TestLogger(TestLog):
    def tearDownModule(self):
        self.logging.shutdown()

    def test_create_verbose_logger(self):
        logger = log.create_logger(self.v_args)
        logger.handlers.clear()

    def test_create_logger(self):
        logger = log.create_logger(self.args)
        logger.handlers.clear()

    def test_one_logger(self):
        logger = log.create_logger(self.v_args)

        with self.assertLogs(logger, level="DEBUG") as cm:
            logger.debug("test")
        self.assertNotRegex(str(cm.output), r".*test.*test")

        logger.handlers.clear()


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

import os
import random
import string
import unittest
import unittest.mock

from waflibs import config, database, error
from waflibs.dns import mysql


def random_string():
    return "".join(
        random.choice(string.ascii_lowercase) for i in range(random.randint(10, 51))
    )


class TestMysql(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.db = unittest.mock.MagicMock()
        cls.ignored_ips = ["1.7.3.0/22"]
        cls.domains = ["example.com"]


class TestAddDnsToMysql(TestMysql):
    def test_wikipedia(self):
        mysql.add_dns_to_mysql(
            self.db,
            random_string(),
            self.ignored_ips,
            self.domains,
            wikipedia_link=random_string(),
        )

    def test_valid_hostname(self):
        mysql.add_dns_to_mysql(
            self.db,
            random_string(),
            self.ignored_ips,
            self.domains,
        )

    def test_no_hostname(self):
        with self.assertRaises(error.ValidationError) as e:
            mysql.add_dns_to_mysql(
                self.db,
                None,
                self.ignored_ips,
                self.domains,
            )

        self.assertRegex(e.exception.message, r"hostname")

    def test_no_domains(self):
        mysql.add_dns_to_mysql(
            self.db,
            random_string(),
            self.ignored_ips,
        )

    def test_add_cname(self):
        mysql.add_dns_to_mysql(
            self.db,
            random_string(),
            self.ignored_ips,
            record_type="CNAME",
            content=random_string(),
        )

    def test_add_existing_cname(self):
        string = random_string()
        name = "EXISTINGCNAME"
        mysql.add_dns_to_mysql(
            self.db,
            name,
            self.ignored_ips,
            record_type="CNAME",
            content=string,
        )
        cursor = self.db.cursor(buffered=True, dictionary=True)
        cursor.execute(
            "SELECT * FROM non_numeric_records \
                WHERE name = '{}'".format(
                name
            )
        )
        results = cursor.fetchall()
        self.assertEqual(len(results), 0)



#!/usr/bin/env/python

import os
import unittest
import unittest.mock

from waflibs import config, database


class TestDatabase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        creds = {
            "host": "blah",
            "username": "blah",
            "password": "blah",
            "database": "blah",
        }
        cls.config = {
            "postgres_credentials": creds,
            "mysql_credentials": creds,
        }


class TestMysql(TestDatabase):
    @unittest.mock.patch("mysql.connector.connect")
    def test_mysql_connect(self, external):
        mysql_config = self.config["mysql_credentials"]

        database.mysql_connect(
            mysql_config["host"],
            mysql_config["username"],
            mysql_config["password"],
            mysql_config["database"],
        )


class TestPostgresql(TestDatabase):
    @unittest.mock.patch("psycopg.connect")
    def test_postgresql_connect(self, external):
        postgres_config = self.config["postgres_credentials"]

        database.postgresql_connect(
            postgres_config["host"],
            postgres_config["username"],
            postgres_config["password"],
            postgres_config["database"],
        )


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
"""utils for arg_parse"""

import argparse
import os
import sys


def create_argument_parser(
    description=None,
    prog=None,
):
    if not prog:
        prog = os.path.basename(sys.argv[0])

    return argparse.ArgumentParser(
        description=description,
        prog=prog,
    )


create_arg_parser = create_argument_parser
create_parser = create_argument_parser


def use_config_file(parser, filename, help_text=None):
    """enable argument for passing in config file"""

    if not help_text:
        help_text = f"Config file (default: %(default)s)"

    parser.add_argument(
        "-c",
        "--config",
        "--config-file",
        type=str,
        default=filename,
        help=help_text,
    )


enable_config = use_config_file
enable_config_file = use_config_file
use_config = use_config_file


def enable_verbose_logging(parser, count=False):
    """enable verbose logger"""

    other_args = {}
    if count:
        action = "count"
        other_args = {
            "default": 0,
        }
    else:
        action = "store_true"

    parser.add_argument(
        "-d",
        "-v",
        "--verbose",
        "--debug",
        action=action,
        help="verbose logging",
        **other_args,
    )


use_verbose_logger = enable_verbose_logging
use_verbose_logging = enable_verbose_logging
use_logger = enable_verbose_logging
enable_logger = enable_verbose_logging
enable_verbose_logger = enable_verbose_logging


def enable_dry_run(parser, count=False, help_text=None):
    """enable dry run mode"""

    if not help_text:
        help_text = "dry run"

    if count:
        action = "count"
    else:
        action = "store_true"

    parser.add_argument(
        "-r",
        "--dry-run",
        "--dryrun",
        action=action,
        help=help_text,
        default=0,
    )


use_dry_run = enable_dry_run



"""
general utils
"""

import os
import shutil
import sys

import dns.resolver
from waflibs.dns import bind, mysql


def get_cloudflare_credentials(config, real_domain=None):
    """get cloudflare credentials"""

    if real_domain:
        domain = real_domain.replace(".", "_")

        return config["cloudflare_credentials"][domain]
    else:
        return config["cloudflare_credentials"]["default"]


def get_default_domains(config, zone_type="public"):
    """get default domains"""

    try:
        zones = config["zones"]["default_domains"]
    except (KeyError, TypeError) as e:
        logger.debug(f"key error - {e}")
        zones = {}

    if zone_type in zones:
        return zones[zone_type]
    else:
        return DEFAULT_DOMAINS


get_cloudflare_config = get_cloudflare_credentials


def convert_origin(orig_name, orig_domain, shared=False):
    """convert domain origin to standard form"""

    full_name = orig_name.strip(".").strip()
    domain = orig_domain.strip(".").strip()

    if shared:
        return full_name.replace(domain, "")

    if full_name == domain:
        return "{}.".format(full_name)
    if full_name == "@":
        return "{}.".format(domain)

    split_name = full_name.split(".")[0:-2]
    if not split_name:
        return "{}.{}.".format(full_name, domain)
    else:
        hostname = ".".join(split_name)
        if domain:
            return "{}.{}.".format(full_name, domain)
        else:
            return "{}.".format(hostname)


def print_cloudflare_errors(error):
    """print full cloudflare errors"""

    if len(error) > 0:
        for err in error:
            print(err)
    else:
        print(error)


print_cf_error = print_cloudflare_errors
print_cf_errors = print_cloudflare_errors


def dns_lookup(record, record_type):
    """lookup dns record"""

    return dns.resolver.query(record, record_type)


def get_domain(hostname):
    logger.debug(f"hostname: {hostname}")

    return ".".join(hostname.split(".")[-2:])



#!/usr/bin/env python
"""mysql manipulation of dns records"""

import ipaddress

from waflibs import dns, error, log, utils

default_bind_dir = "/etc/bind"

logger = log.logger().get_logger()


def increment_ips(
    db,
    ipv6_address,
    ip_address,
    no_auto_ipv6_address,
    no_auto_ip_address,
    ignored_ips,
):
    if no_auto_ipv6_address:
        logger.debug("ipv6 address manually set.")
        logger.debug("not auto incrementing ipv6 address.")
        logger.debug("ipv6 address being used: {}".format(ipv6_address))
    else:
        logger.debug("ipv6 address not manually set.")
        logger.debug("auto incrementing ipv6 address.")
        command = "SELECT INET6_NTOA(max(ipv6)) FROM numeric_records \
                WHERE INET6_NTOA(ipv6) LIKE '2607:5300:60:3b25%';"
        logger.debug("command to execute: {}".format(command))
        cursor = db.cursor()
        cursor.execute(command)

        lowest_ipv6_address = cursor.fetchone()[0]
        cursor.close()

        logger.debug("lowest ipv6 address: {}".format(lowest_ipv6_address))
        if lowest_ipv6_address:
            ipv6_address = str(ipaddress.ip_address(lowest_ipv6_address) + 1)
        else:
            ipv6_address = None

    if no_auto_ip_address:
        logger.debug("ip address manually set.")
        logger.debug("not auto incrementing ip address.")
        logger.debug("ip address being used: {}".format(ip_address))
    else:
        logger.debug("ip address not manually set.")

        if ignored_ips:
            logger.debug("ignored ips array: {}".format(ignored_ips))
            ignored_ips_string = " ".join(
                [f"AND INET_NTOA(ip) NOT LIKE '{ii}.%'" for ii in ignored_ips]
            )[3:]
            ignored_ips_string = f"WHERE {ignored_ips_string}"
            logger.debug("ignored ip string: {}".format(ignored_ips_string))
        else:
            ignored_ips_string = ""

        min_count = f"SELECT MIN(cnt) INTO @min FROM (SELECT COUNT(*) cnt \
                FROM numeric_records {ignored_ips_string} GROUP BY ip) t;"

        min_ip = "SELECT INET_NTOA(t1.ip) FROM numeric_records t1 JOIN \
                (SELECT ip FROM numeric_records GROUP BY ip \
                HAVING COUNT(*) = @min) t2 ON t1.ip = t2.ip;"

        logger.debug("command to execute: {}".format(min_count))
        cursor = db.cursor(buffered=True)
        cursor.execute(min_count)
        logger.debug("command to execute: {}".format(min_ip))
        cursor.execute(min_ip)

        lowest_ip_address = cursor.fetchone()[0]
        cursor.close()

        logger.debug("least used ip address: {}".format(lowest_ip_address))
        if lowest_ip_address:
            ip_address = str(lowest_ip_address)
        else:
            ip_address = None

        logger.debug("auto incrementing ip address.")

    return ipv6_address, ip_address


def add_dns_to_mysql(
    db,
    hostname,
    ignored_ips=None,
    domains=None,
    content=None,
    record_type=None,
    ipv6_address=None,
    ip_address=None,
    priority=0,
    ttl=0,
    proxied=True,
    wikipedia_link=None,
    no_auto_ip_address=False,
    no_auto_ipv6_address=False,
    dry_run=False,
    public=False,
):
    if not hostname:
        message = "you must specify a hostname"
        logger.error(message)

        raise error.ValidationError(message)

    if not record_type or record_type == "AAAA" or record_type == "A":
        ipv6_address, ip_address = increment_ips(
            db,
            ipv6_address,
            ip_address,
            no_auto_ipv6_address,
            no_auto_ip_address,
            ignored_ips,
        )

    if wikipedia_link:
        wikipedia_link = wikipedia_link.strip()
        logger.debug("wikipedia link: {}".format(wikipedia_link))
        if "wikipedia" not in wikipedia_link:
            raise Exception("not a wikipedia link")

        command = "INSERT INTO dbm_hostnames (hostname, url) VALUES \
                ('{}', '{}');".format(
            hostname, wikipedia_link
        )
        logger.debug("command to execute: {}".format(command))
        if dry_run:
            logger.info("would execute mysql command")

            result = -1
        else:
            logger.debug("executing mysql command")
            cursor = db.cursor()
            result = cursor.execute(command)
            cursor.close()

    logger.debug("domains: {}".format(domains))
    if domains is None:
        if public:
            actual_domains = ["ALL_PUBLIC"]
        else:
            actual_domains = ["ALL_PRIVATE"]
    else:
        actual_domains = domains
    logger.debug("actual domains: {}".format(actual_domains))
    for domain in actual_domains:
        if record_type:
            logger.debug(f"record type orig: {record_type}")
            record_type = record_type.upper()
            logger.debug(f"record type: {record_type}")
            if content:
                content = content.strip()
                logger.debug("content: {}".format(content))
                priority = priority
                logger.debug("record priority: {}".format(priority))

                command = "INSERT INTO non_numeric_records (name, domain, \
                        type, content, priority, ttl, proxied) \
                        VALUES ('{}', '{}', '{}', '{}', {}, {}, {});".format(
                    hostname,
                    domain,
                    record_type,
                    content,
                    int(priority),
                    int(ttl),
                    int(proxied),
                )

                dupe_command = "SELECT * FROM non_numeric_records \
                        WHERE name = '{}' AND domain = '{}' \
                        AND type = '{}' AND content = '{}';".format(
                    hostname,
                    domain,
                    record_type,
                    content,
                )
            else:
                raise RuntimeError("you must specify content for this type")
        else:
            if (ip_address and ipv6_address) or (not ip_address and not ipv6_address):
                command = "INSERT INTO numeric_records (hostname, domain, ip, \
                        ipv6, proxied, ttl) VALUES ('{}', '{}', \
                        INET_ATON('{}'), INET6_ATON('{}'), {}, {});".format(
                    hostname,
                    domain,
                    ip_address,
                    ipv6_address,
                    int(proxied),
                    ttl,
                )

                dupe_command = "SELECT * FROM numeric_records \
                        WHERE hostname = '{}' AND domain = '{}' \
                        AND ip = INET_ATON('{}') \
                        AND ipv6 = INET6_ATON('{}');".format(
                    hostname,
                    domain,
                    ip_address,
                    ipv6_address,
                )
            elif ip_address and not ipv6_address:
                command = "INSERT INTO numeric_records \
                        (hostname, domain, ip, ipv6, proxied, ttl) \
                        VALUES ('{}', '{}', INET_ATON('{}'), NULL, \
                        {}, {});".format(
                    hostname,
                    domain,
                    ip_address,
                    int(proxied),
                    ttl,
                )

                dupe_command = "SELECT * FROM numeric_records \
                        WHERE hostname = '{}' AND domain = '{}' \
                        AND ip = INET_ATON('{}');".format(
                    hostname, domain, ip_address
                )
            elif ipv6_address and not ip_address:
                command = "INSERT INTO numeric_records \
                        (hostname, domain, ip, ipv6, proxied, ttl) \
                        VALUES ('{}', '{}', NULL, INET6_ATON('{}'), \
                        {}, {});".format(
                    hostname, domain, ipv6_address, int(proxied), ttl
                )

                dupe_command = "SELECT * FROM numeric_records \
                        WHERE hostname = '{}' AND domain = '{}' \
                        AND ipv6 INET6_ATON('{}');".format(
                    hostname,
                    domain,
                    ipv6_address,
                )
            else:
                raise RuntimeError("This should never happen")
        if dry_run:
            logger.info("would execute mysql command")

            results = -1
        else:
            duplicate = False
            logger.debug("executing mysql command")
            cursor = db.cursor(buffered=True, dictionary=True)

            special_types = ["CNAME"]
            if record_type in special_types:
                dupe_command = "SELECT * FROM non_numeric_records \
                        WHERE name = '{}' AND domain = '{}' \
                        AND type = '{}';".format(
                    hostname, domain, record_type
                )
            logger.debug("check duplicate command: {}".format(dupe_command))
            cursor.execute(dupe_command)
            results = cursor.fetchone()
            cursor.close()
            if results:
                duplicate = True
            if duplicate:
                if record_type in special_types:
                    command = "UPDATE non_numeric_records \
                            SET content = '{content}', \
                            name = '{name}', \
                            domain = '{domain}', \
                            type = '{record_type}', \
                            priority = {priority}, \
                            ttl = {ttl}, \
                            proxied = {proxied} \
                            WHERE name = '{name}' AND \
                            domain = '{domain}' AND \
                            type = '{record_type}' AND \
                            priority = {priority} AND \
                            ttl = {ttl} AND \
                            proxied = {proxied};".format(
                        name=hostname,
                        domain=domain,
                        record_type=record_type,
                        content=content,
                        priority=int(priority),
                        ttl=int(ttl),
                        proxied=int(proxied),
                    )
                    logger.debug("command to execute: {}".format(command))
                    cursor = db.cursor()
                    logger.debug("executing mysql command")
                    cursor.execute(command)
                    cursor.close()
                else:
                    logger.info("record already exists. not doing anything")
            else:
                logger.debug("command to execute: {}".format(command))
                cursor = db.cursor()
                logger.debug("executing mysql command")
                cursor.execute(command)
                cursor.close()

        if ipv6_address:
            ipv6_address = str(ipaddress.ip_address(ipv6_address) + 1)

    logger.debug("all db results: {}".format(str(cursor)))

    db.commit()


def generate_bind_file(
    db,
    zones,
    bind_dir=default_bind_dir,
    serial=None,
    dry_run=False,
):
    cursor = db.cursor()

    if zones:
        if isinstance(zones, str):
            domains = [zones]
        else:
            domains = zones
    else:
        command = "SELECT DISTINCT domain from non_numeric_records;"
        cursor.execute(command)
        nnr_domains = tuple(map(lambda l: l[0], cursor.fetchall()))
        command = "SELECT DISTINCT domain from numeric_records;"
        cursor.execute(command)
        nr_domains = tuple(map(lambda l: l[0], cursor.fetchall()))
        domains = set(nnr_domains + nr_domains)
    logger.debug("distinct domains to generate: {}".format(str(domains)))

    if serial:
        serial_number = int(serial)
    else:
        serial_number = dns.bind.generate_serial_number()

    for domain in domains:
        dns.bind.generate_bind_file(
            get_records(cursor, domain),
            domain,
            bind_dir=bind_dir,
            serial_number=serial_number,
            dry_run=dry_run,
            template_file_name="{}/db.template".format(bind_dir),
            domain_template_file_name="{}/db.{}.template".format(bind_dir, domain),
        )

    for shared_name in ["ALL_PRIVATE", "ALL_PUBLIC"]:
        dns.bind.generate_bind_file(
            get_records(cursor, shared_name, shared=True),
            shared_name,
            bind_dir=bind_dir,
            zone_file_name="db.shared.{}".format(shared_name.split("_")[-1].lower()),
            serial_number=serial_number,
            dry_run=dry_run,
        )


def get_records(cursor, domain, shared=False):
    logger.debug("starting to get records for {}".format(domain))

    # get all records from the db
    command = "SELECT * FROM non_numeric_records WHERE domain = '{}';".format(domain)
    logger.debug("about to execute sql: {}".format(command))
    cursor.execute(command)
    nnr_results = cursor.fetchall()
    command = "SELECT hostname,domain,INET_NTOA(ip),INET6_NTOA(ipv6),proxied,ttl FROM \
        numeric_records WHERE domain = '{}';".format(
        domain
    )
    logger.debug("about to execute sql: {}".format(command))
    cursor.execute(command)
    nr_results = cursor.fetchall()
    all_results = {"numeric": nr_results, "non_numeric": nnr_results}
    logger.debug("all db results: {}".format(str(all_results)))

    all_records = []
    while all_results:
        result_type, results = all_results.popitem()
        logger.debug(f"result type: {result_type}")

        if result_type == "numeric":
            for row in results:
                logger.debug("numeric row result: {}".format(str(row)))

                short_hostname, domain, ip, ipv6, proxied, ttl = row

                logger.debug("original hostname: {}".format(short_hostname))
                hostname = utils.convert_origin(short_hostname, domain, shared)

                logger.debug("hostname: {}, domain: {}".format(hostname, domain))
                logger.debug("ip: {}, ipv6: {}".format(ip, ipv6))

                if ip:
                    all_records.append(
                        {
                            "name": hostname,
                            "type": "A",
                            "content": ip,
                            "proxied": proxied,
                        }
                    )
                if ipv6:
                    all_records.append(
                        {
                            "name": hostname,
                            "type": "AAAA",
                            "content": ipv6,
                            "proxied": proxied,
                        }
                    )
        elif result_type == "non_numeric":
            for row in results:
                logger.debug("non-numeric row result: {}".format(str(row)))

                (
                    _id,
                    short_name,
                    domain,
                    record_type,
                    priority,
                    content,
                    proxied,
                    ttl,
                ) = row

                logger.debug("original name: {}".format(short_name))
                name = utils.convert_origin(short_name, domain, shared)

                logger.debug(
                    "name: {}, domain: {}, record_type: {},".format(
                        name, domain, record_type
                    )
                )
                logger.debug("content: {}, proxied: {}".format(content, proxied))

                all_records.append(
                    {
                        "name": name,
                        "type": record_type,
                        "content": content,
                        "proxied": proxied,
                    }
                )
        else:
            raise Exception("this should not happen")

    logger.debug("finished getting records for {}".format(domain))

    return all_records



#!/usr/bin/env python
"""utils for manipulating text"""


def idna(domain_name):
    """convert unicode domain name to punycode"""

    return domain_name.encode("idna").decode("utf-8")


idn = idna
i_d_n = idna
i_d_na = idna



#!/usr/bin/env python

import datetime
import tempfile
import unittest

from waflibs.dns import bind


class TestDns(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        bind_dir = tempfile.TemporaryDirectory()

        cls.serial = int(datetime.datetime.now().strftime("%Y%m%d00"))
        cls.bind_dir = bind_dir
        cls.domain = "test.com"
        cls.records = [
            {
                "name": "fullname.record.",
                "type": "CNAME",
                "content": "nodot",
                "proxied": False,
            },
            {
                "name": "shortname",
                "type": "CNAME",
                "content": "hasadot.",
                "proxied": False,
            },
            {
                "name": "test",
                "type": "A",
                "content": "1.2.3.4",
                "proxied": False,
            },
            {
                "name": "longtest.",
                "type": "AAAA",
                "content": "123:ABC::4",
                "proxied": False,
            },
            {
                "name": "sigh.test.com.",
                "type": "A",
                "content": "2.3.4.5",
                "proxied": False,
            },
            {
                "name": "test.com.",
                "type": "MX",
                "content": "mx.garbage",
                "proxied": False,
                "priority": 99,
            },
            {
                "name": "nameserver",
                "type": "NS",
                "content": "name.server.",
                "proxied": False,
            },
        ]
        cls.result = """; HEADER

$ORIGIN test.com

{}

fullname.record. IN CNAME nodot
shortname IN CNAME hasadot.
test IN A 1.2.3.4
longtest. IN AAAA 123:ABC::4
sigh.test.com. IN A 2.3.4.5
test.com. IN MX 99 mx.garbage
nameserver IN NS name.server.

; 7 total records found

"""

    @classmethod
    def tearDownClass(cls):
        cls.bind_dir.cleanup()

    def setUp(self):
        tf = tempfile.NamedTemporaryFile()
        tf.write(b"; HEADER\n\n$ORIGIN {zone}\n\n{serial}\n")
        tf.seek(0)

        self.template_file = tf

        dtf = tempfile.NamedTemporaryFile()
        dtf.write(b"{records}\n")
        dtf.seek(0)

        self.domain_template_file = dtf

        s = open(f"{self.bind_dir.name}/serial", "w")
        s.write(str(self.serial))
        s.write("\n")
        s.seek(0)

        self.serial_file = s

    def test_generate_serial(self):
        serial_filename = tempfile.NamedTemporaryFile()

        serial = bind.generate_serial(
            serial_number=self.serial,
            serial_file=serial_filename.name,
        )
        self.assertEqual(serial, self.serial)

    def test_generate_serial_filename(self):
        serial_filename = tempfile.NamedTemporaryFile()

        serial = bind.generate_serial(
            serial_file=serial_filename.name,
        )
        self.assertEqual(serial, self.serial)

    def test_generate_bind_file_serial(self):
        zone_file_name = bind.generate_bind_file(
            self.records,
            self.domain,
            serial_number=self.serial,
            serial_file=self.serial_file.name,
            bind_dir=self.bind_dir.name,
            template_file_name=self.template_file.name,
            domain_template_file_name=self.domain_template_file.name,
        )

        with open(zone_file_name) as f:
            self.assertEqual(f.read(), self.result.format(self.serial))

    def test_generate_bind_file_no_serial(self):
        zone_file_name = bind.generate_bind_file(
            self.records,
            self.domain,
            bind_dir=self.bind_dir.name,
            template_file_name=self.template_file.name,
            domain_template_file_name=self.domain_template_file.name,
            serial_file=self.serial_file.name,
        )

        with open(zone_file_name) as f:
            self.assertEqual(f.read(), self.result.format(self.serial + 1))


if __name__ == "__main__":
    unittest.main()
